// Based on an example kindly provided by George Christman and Lance Java.

package jumpstart.business.domain.examples;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import javax.ejb.Local;
import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import jumpstart.business.domain.examples.iface.CategoryNode;
import jumpstart.business.domain.examples.iface.ICategoryServiceLocal;
import jumpstart.business.domain.examples.iface.ICategoryServiceRemote;

@Stateless
@Local(ICategoryServiceLocal.class)
@Remote(ICategoryServiceRemote.class)
public class CategoryService implements ICategoryServiceLocal, ICategoryServiceRemote {

	@PersistenceContext(unitName = "jumpstart")
	private EntityManager em;

	public Category findCategory(Integer id) {
		return em.find(Category.class, id);
	}

	public CategoryNode findCategoryInfo(Integer id) {
		List<Category> categories = new ArrayList<Category>();
		categories.add(findCategory(id));
		return getCategoryInfo(categories).iterator().next();
	}

	public List<CategoryNode> getChildren(CategoryNode node) {
		List<Category> categories = findChildren(node.getCategory().getId());
		return getCategoryInfo(categories);
	}

	public List<CategoryNode> findRoots() {
		List<Category> categories = findCategoriesWithNoParent();
		if (categories == null) {
			return new ArrayList<CategoryNode>();
		}
		return getCategoryInfo(categories);
	}

	@SuppressWarnings("unchecked")
	private List<Category> findChildren(Integer id) {
		return em.createQuery("select c from Category c where c.parent.id = :id").setParameter("id", id)
				.getResultList();
	}

	@SuppressWarnings("unchecked")
	private List<Category> findCategoriesWithNoParent() {
		return em.createQuery("select c from Category c where c.parent is null").getResultList();
	}

	/**
	 * Given a list of Categories, returns a list of CategoryNodes. Each one represents a Category and some info about
	 * the Category: specifically whether it has children or is a leaf.
	 */
	@SuppressWarnings("unchecked")
	private List<CategoryNode> getCategoryInfo(List<Category> categories) {
		Map<Integer, CategoryNode> categoryNodesById = new LinkedHashMap<Integer, CategoryNode>();

		// Build a map of skeleton CategoryNodes by Category id - one entry per given Category.

		for (Category category : categories) {
			CategoryNode categoryNode = new CategoryNode();
			categoryNode.setCategory(category);
			categoryNodesById.put(category.getId(), categoryNode);
		}

		if (!categoryNodesById.isEmpty()) {

			// Query whether each Category has children.

			StringBuilder buf = new StringBuilder();

			// This JPQL query should have worked but Hibernate translates the count to "count(.)" which is invalid SQL
			// (a Hibernate bug?)...
			// buf.append("select c1.id, count(c1.children) from Category c1");
			// buf.append(" where c1.id in (:catIds) ");
			// buf.append(" group by c1.id");
			// Query q = em.createQuery(buf.toString());

			// ...so we use a native query instead
			buf.append("select c1.id, count(c2.id) from Category c1");
			buf.append(" left join Category c2 on c2.parentId = c1.id");
			buf.append(" where c1.id in (:catIds) ");
			buf.append(" group by c1.id");
			Query q = em.createNativeQuery(buf.toString());

			q.setParameter("catIds", categoryNodesById.keySet());
			List<Object[]> l = q.getResultList();

			// Update each Category's corresponding CategoryNode in the map with whether it has children or is a leaf.

			for (Object[] result : l) {
				Integer categoryId = (Integer) result[0];
				int childCount = ((Number) result[1]).intValue();

				CategoryNode categoryNode = categoryNodesById.get(categoryId);
				categoryNode.setHasChildren(childCount != 0);
				categoryNode.setIsLeaf(childCount == 0);
			}
		}

		return new ArrayList<CategoryNode>(categoryNodesById.values());
	}

}
