package jumpstart.business.domain.examples;

import java.util.List;

import javax.ejb.Local;
import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.EntityNotFoundException;
import javax.persistence.OptimisticLockException;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import jumpstart.business.commons.IdVersion;
import jumpstart.business.domain.examples.iface.IPersonServiceLocal;
import jumpstart.business.domain.examples.iface.IPersonServiceRemote;
import jumpstart.business.domain.examples.iface.PersonDTO;

@Stateless
@Local(IPersonServiceLocal.class)
@Remote(IPersonServiceRemote.class)
public class PersonService implements IPersonServiceLocal, IPersonServiceRemote {

	@PersistenceContext(unitName = "jumpstart")
	private EntityManager em;

	public Person findPerson(Long id) {
		return em.find(Person.class, id);
	}

	@SuppressWarnings("unchecked")
	public List<Person> findPersons(int maxResults) {
		return em.createQuery("select p from Person p order by lower(p.firstName), lower(p.lastName)")
				.setMaxResults(maxResults).getResultList();
	}

	@SuppressWarnings("unchecked")
	public List<Person> findPersons(String partialName, int maxResults) {
		String searchName = partialName == null ? "" : partialName.toLowerCase();

		StringBuilder buf = new StringBuilder();
		buf.append("select p from Person p");
		buf.append(" where lower(firstName) like :firstName");
		buf.append(" or lower(lastName) like :lastName");
		buf.append(" order by lower(p.firstName), lower(p.lastName)");

		Query q = em.createQuery(buf.toString());
		q.setParameter("firstName", "%" + searchName + "%");
		q.setParameter("lastName", "%" + searchName + "%");
		q.setMaxResults(maxResults);

		List<Person> l = q.getResultList();
		return l;
	}

	@SuppressWarnings("unchecked")
	public List<Person> findPersonsByFirstName(String firstName) {
		String searchName = firstName == null ? "" : firstName.trim().toLowerCase();

		StringBuilder buf = new StringBuilder();
		buf.append("select p from Person p");
		buf.append(" where lower(p.firstName) = :searchName");
		buf.append(" order by lower(p.firstName), lower(p.lastName)");

		Query q = em.createQuery(buf.toString());
		q.setParameter("searchName", searchName);

		List<Person> l = q.getResultList();
		return l;
	}

	@SuppressWarnings("unchecked")
	public List<Person> findPersonsByLastName(String lastName) {
		String searchName = lastName == null ? "" : lastName.trim().toLowerCase();

		StringBuilder buf = new StringBuilder();
		buf.append("select p from Person p");
		buf.append(" where lower(p.lastName) = :searchName");
		buf.append(" order by lower(p.lastName), lower(p.firstName)");

		Query q = em.createQuery(buf.toString());
		q.setParameter("searchName", searchName);

		List<Person> l = q.getResultList();
		return l;
	}

	public Person createPerson(Person person) {
		em.persist(person);
		return person;
	}

	public void createPersons(List<Person> persons) {
		for (Person person : persons) {
			em.persist(person);
		}
	}

	public void changePerson(Person person) {
		Person p = em.merge(person);
		// Flush to work around OPENEJB issue https://issues.apache.org/jira/browse/OPENEJB-782
		em.flush();

		// If id is different it means the person did not exist so merge has created a new one.
		if (!p.getId().equals(person.getId())) {
			throw new EntityNotFoundException("Person no longer exists.");
		}
	}

	public void changePersons(List<Person> persons) {
		for (Person person : persons) {
			Person p = em.merge(person);

			// If id is different it means the person did not exist so merge has created a new one.
			if (!p.getId().equals(person.getId())) {
				throw new EntityNotFoundException("Person no longer exists.");
			}
		}
	}

	public void changePersonsByDTOs(List<PersonDTO> persons) {
		for (PersonDTO person : persons) {
			Person p = em.find(Person.class, person.getId());

			if (p == null) {
				throw new EntityNotFoundException("Person no longer exists.");
			}

			if (!p.getVersion().equals(person.getVersion())) {
				throw new OptimisticLockException();
			}

			p.setFirstName(person.getFirstName());
		}
	}

	public void bulkEditPersons(List<Person> personsToCreate, List<Person> personsToChange,
			List<IdVersion> personsToDelete) {
		for (Person person : personsToCreate) {
			em.persist(person);
		}
		for (Person person : personsToChange) {
			Person p = em.merge(person);

			// If id is different it means the person did not exist so merge has created a new one.
			if (!p.getId().equals(person.getId())) {
				throw new EntityNotFoundException("Person no longer exists.");
			}
		}
		for (IdVersion idVersion : personsToDelete) {
			Person p = em.find(Person.class, idVersion.getId());

			if (p == null) {
				throw new EntityNotFoundException("Person no longer exists.");
			}

			if (!p.getVersion().equals(idVersion.getVersion())) {
				throw new OptimisticLockException();
			}

			em.remove(p);
		}
	}

	public void bulkEditPersonsByDTOs(List<PersonDTO> personsToCreate, List<PersonDTO> personsToChange,
			List<IdVersion> personsToDelete) {
		for (PersonDTO p : personsToCreate) {
			Person person = new Person(p.getFirstName(), p.getLastName(), p.getRegion(), p.getStartDate());
			em.persist(person);
		}
		for (PersonDTO person : personsToChange) {
			Person p = em.find(Person.class, person.getId());

			if (p == null) {
				throw new EntityNotFoundException("Person no longer exists.");
			}

			if (!p.getVersion().equals(person.getVersion())) {
				throw new OptimisticLockException();
			}

			p.setFirstName(person.getFirstName());
			p.setLastName(person.getLastName());
			p.setRegion(person.getRegion());
			p.setStartDate(person.getStartDate());
		}
		for (IdVersion idVersion : personsToDelete) {
			Person p = em.find(Person.class, idVersion.getId());

			if (p == null) {
				throw new EntityNotFoundException("Person no longer exists.");
			}

			if (!p.getVersion().equals(idVersion.getVersion())) {
				throw new OptimisticLockException();
			}

			em.remove(p);
		}
	}

	public void deletePerson(Long id, Integer version) {
		Person p = em.find(Person.class, id);

		if (p == null) {
			throw new EntityNotFoundException("Person no longer exists.");
		}

		if (!p.getVersion().equals(version)) {
			throw new OptimisticLockException();
		}

		em.remove(p);
		// Flush to work around OPENEJB issue https://issues.apache.org/jira/browse/OPENEJB-782
		em.flush();
	}

}
